<?php

class DoceboApi {

	private static $_instance;

	protected $_url;
	protected $_key;
	protected $_secretKey;
	protected $_sso;

	public function getUrl() {
		return $this->_url;
	}
	
	public function getKey() {
		return $this->_key;
	}
	
	public static function getInstance() {
		if (!is_object(self::$_instance)) {
			self::$_instance = new DoceboApi();
		}

		return self::$_instance;
	}

	public function __construct() {
		$this->_url = variable_get('docebo_lms_cloud_address', '');
		$this->_key = variable_get('docebo_lms_key', '');
		$this->_secretKey = variable_get('docebo_lms_secret_key', '');
		$this->_sso = variable_get('docebo_lms_sso', '');
	}

	private function getHash($params = array()) {
		$hashes = array(
			'sha1' => sha1(implode(',', $params) . ',' . $this->_secretKey),
		);

		$hashes['x_auth'] = base64_encode($this->_key . ':' . $hashes['sha1']);

		return $hashes;
	}

	private function getHeaders($xAuthHash) {
		return array(
			'Host: ' . $this->_url,
			'Content-Type: multipart/form-data',
			'X-Authorization: Docebo ' . $xAuthHash,
		);
	}

	private function curlOptions($optArray = array()) {
		$this->_curl = curl_init();

		$options = array(
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_POST => true,
			CURLOPT_CONNECTTIMEOUT => 5,
		);

		$options += $optArray;
		curl_setopt_array($this->_curl, $options);
	}

	public function call($action, $params = array()) {
		if ($action != 'user/apiResponseTest') {
			if ($this->checkConnection() == false) {
				drupal_set_message(t('Docebo SSO configuration is incorrect'), 'error');
				return false;
			}	
		}

		$hash = $this->getHash($params);
		$headers = $this->getHeaders($hash['x_auth']);

		$this->curlOptions(array(
			CURLOPT_URL => $this->_url . '/api/' . $action,
			CURLOPT_POSTFIELDS => $params,
			CURLOPT_HTTPHEADER => $headers,	
		));

		return $this->doAction();
	}

	private function doAction() {
		$output = curl_exec($this->_curl);
		curl_close($this->_curl);

		return json_decode($output);
	}

	public function checkConnection() {
		if (!empty($this->_secretKey) && !empty($this->_key) && !empty($this->_sso)) {
			return $this->call('user/apiResponseTest');
		} else {
			return false;
		}
	}

	public function getStats() {
		global $user;

		if ($user->uid == 0) {
			return array();
		}

		$result = $this->call('user/getstat', array('ext_user_type' => 'drupal', 'ext_user' => $user->uid));

		if (isset($result->success) && $result->success == 1) {
			return array(
				'courses' => $result->user_courses,
				'lastCourse' => $result->last_course,
			);
		} else {
			return array();
		}
	}

	public function getUsersCount() {
		$result = $this->call('user/listUsers', array());

		if (isset($result->success) && $result->success == 1) {
			return count($result->users);
		}

		return false;
	}

	public function syncUsers($users) {
		$users = urlencode(json_encode($users));
		$result = $this->call('user/syncWordpressUsers', array('users' => $users));
		$synced = 0;
		if (isset($result->success) && $result->success == 1) {
			foreach ($result->result as $response) {
				if ($response->success == 1) {
					$synced++;
				}
			}
			return $synced;
		} else {
			return false;
		}
	} 

	public function getSSOLink($params = array()) {
		if ($this->checkConnection() == false) {
			drupal_set_message(t('Docebo SSO configuration is incorrect'), 'error');
			return false;
		}

		if (empty($params)) {
			global $user;

			// A fallback, in case we can't retrieve
			// the LMS username via API. Use the Drupal
			// username of the user instead in this case
			$username = $user->name;

			if(isset($user->uid)){
				$cacheKey = 'docebo_sso_user_pair_'.$user->uid;

				$cachedDoceboUsername = cache_get($cacheKey, 'cache');

				if($cachedDoceboUsername && $cachedDoceboUsername->expire > time()){
					// Cache found
					$username = $cachedDoceboUsername->data;

					watchdog('docebo', 'Retrieved LMS username of current Drupal user from cache. Drupal user: '.$user->uid.', LMS user: '.$username, array(), WATCHDOG_INFO);
				}else{
					$params = array(
						'ext_user' => isset($user->uid) ? $user->uid : null,
						'ext_user_type' => 'drupal',
					);
					$data = DoceboApi::call('user/profile/', $params);

					if($data && isset($data->userid)){
						watchdog('docebo', 'Successfully retrieved LMS username of current Drupal user. Caching the pair...', array(), WATCHDOG_INFO);

						$username = $data->userid;

						// Cache this username pair (for 10 minutes) in case he
						// tries to SSO login soon again
						cache_set($cacheKey, $username, 'cache', time()+(60 * 10));
					}else{
						watchdog('docebo', 'Unable to retrieve LMS username/counterpart of current Drupal user', array(), WATCHDOG_ERROR);
					}
				}

			}else{
				watchdog('docebo', 'Current Drupal user unknown. Can not retrieve LMS username to be included in SSO link without this information.', array(), WATCHDOG_ERROR);
				// Who is the current user?
				return null;
			}

		} else {
			$username = $params['username'];
		}

		$time = time();
		$token = md5($username .','. $time .','. $this->_sso);

		return 'http://' . $this->_url . '/doceboLms/index.php?auth_regen=1&modname=login&op=confirm&login_user=' . $username . '&time=' . $time . '&token=' . $token;
	}

	public function getCourses() {
		global $user;

		if ($this->checkConnection() == false) {
			drupal_set_message(t('Docebo LMS configuration is incorrect'), 'error');
			return false;
		}

		$courses = $this->call('user/userCourses', array('ext_user_type' => 'drupal', 'ext_user' => $user->uid));

		$out = array();
		foreach ((Array) $courses as $key => $course) {
			if (is_numeric($key)) {
				$out[] = array(
					'id' => $course->course_info->course_id,
					'title' => $course->course_info->course_name,
					'description' => $course->course_info->course_description,
					'course_url' => $course->course_info->course_link,
					'image_url' => $course->course_info->course_thumbnail,
					'status' => $course->course_info->courseuser_status == 2 ? 'completed' : 'in-progress',
				);
			}
		}

		return $out;
	}

	public function getAuthUrl($username, $password) {
		$result = $this->call('auth/authenticate', array('username' => $username, 'password' => $password));

		if (!empty($result->success) && $result->success == 1) {
			return array(
				'success' => true,
				'token' => $result->token,
				'url' => $this->getSSOLink(array('username' => $username)),
			);
		} else {
			return array(
				'success' => false,
				'message' => $result->message,
			);
		}
	}
}